# Generated by Django 4.2.7 on 2023-12-04 22:35

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_action_item", "0034_alter_actionitem_device_created_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="actionitem",
            options={
                "default_manager_name": "objects",
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "verbose_name": "Action Item",
                "verbose_name_plural": "Action Items",
            },
        ),
        migrations.AlterModelOptions(
            name="actiontype",
            options={
                "default_manager_name": "objects",
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
            },
        ),
        migrations.AlterModelOptions(
            name="reference",
            options={
                "default_manager_name": "objects",
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
            },
        ),
        migrations.RemoveIndex(
            model_name="actionitem",
            name="edc_action__id_6cb4e0_idx",
        ),
        migrations.AddField(
            model_name="actionitem",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="actionitem",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="actiontype",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="actiontype",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="edcpermissions",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="edcpermissions",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalactionitem",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalactionitem",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalreference",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalreference",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="reference",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="reference",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddIndex(
            model_name="actionitem",
            index=models.Index(
                fields=["modified", "created"], name="edc_action__modifie_d8a9b7_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="actionitem",
            index=models.Index(
                fields=["user_modified", "user_created"], name="edc_action__user_mo_b01b34_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="actionitem",
            index=models.Index(
                fields=["subject_identifier"], name="edc_action__subject_0dae34_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="actionitem",
            index=models.Index(
                fields=[
                    "subject_identifier",
                    "action_identifier",
                    "action_type",
                    "site_id",
                    "status",
                    "report_datetime",
                ],
                name="edc_action__subject_5bacf2_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="actiontype",
            index=models.Index(
                fields=["modified", "created"], name="edc_action__modifie_7ecb65_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="actiontype",
            index=models.Index(
                fields=["user_modified", "user_created"], name="edc_action__user_mo_62771e_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="reference",
            index=models.Index(
                fields=["modified", "created"], name="edc_action__modifie_17f7a2_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="reference",
            index=models.Index(
                fields=["user_modified", "user_created"], name="edc_action__user_mo_e27ce1_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="reference",
            index=models.Index(
                fields=["subject_identifier"], name="edc_action__subject_b074e8_idx"
            ),
        ),
    ]
